package com.hikcreate.update_hbase.controller;


import com.hikcreate.update_hbase.service.EveryDayScheduleService;
import com.hikcreate.update_hbase.service.VehicleIndexService;
import com.hikcreate.update_hbase.service.VioIndexService;
import com.hikcreate.update_hbase.service.impl.VehicleIndexServiceImpl;
import com.hikcreate.update_hbase.service.impl.VioIndexServiceImpl;
import com.hikcreate.update_hbase.utils.HbaseUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

@RestController
@RequestMapping(path = "/dataStation")
public class OracleToHbaseController {
    private Logger logger = LoggerFactory.getLogger(RestController.class);

    @Autowired
    private VehicleIndexService vehicleIndexService;
    @Autowired
    private VioIndexService vioIndexService;
    @Autowired
    private EveryDayScheduleService everyDayScheduleService;

    /**
     * 从Oracle导入全量机动车索引数据到hbase
     */
    @RequestMapping(path = "/insertVehicleIndex/all")
    public void insertVehicleIndexAll() {
        vehicleIndexService.listVehIndexall();
    }

    /**
     * 从Oracle导入昨日机动车索引数据到hbase
     */
    @RequestMapping(path = "/insertVehicleIndex/yesterday")
    public void insertVehicleIndexYesterday() {
        vehicleIndexService.listVehIndexyesterday();
    }


    /**
     * 从Oracle导入全量违法索引数据到hbase
     */
    @RequestMapping(path = "/insertVioIndex/all")
    public void insertVioIndexAll() {
        vioIndexService.listVioIndexAll();
    }


    /**
     * 从Oracle导入昨日违法索引数据到hbase
     */
    @RequestMapping(path = "/insertVioIndex/yesterday")
    public void insertVioIndexYesterday() {
        vioIndexService.listVioIndexYesterday();
    }

    /**
     * 从Oracle导入几天前的当日数据 ep：days=1 表示导入昨日数据，插入hbase ，发Kafka
     *
     * @param days 天
     */
    @RequestMapping(path = "/insertVio/DaysAgo")
    public void insertVioDaysAgo(@RequestParam("days") int days) {
        everyDayScheduleService.getViolationData(days);
    }

    /**
     * 从Oracle导入几天前的当日数据 ep：days=1 表示导入昨日数据，插入hbase ，发Kafka
     *
     * @param days 天
     */
    @RequestMapping(path = "/insertVeh/DaysAgo")
    public void insertVehDaysAgo(@RequestParam("days") int days) {
        everyDayScheduleService.getVehicleData(days);
    }

    /**
     * 从Oracle导入几天前的当日数据 ep：days=1 表示导入昨日数据，插入hbase ，发Kafka
     *
     * @param days 天
     */
    @RequestMapping(path = "/insertDrv/DaysAgo")
    public void insertDrvDaysAgo(@RequestParam("days") int days) {
        everyDayScheduleService.getDrivinglicenseData(days);
    }

    /**
     * 从Oracle导入 days天前的当日数据
     *
     * @param days 天
     */
    @RequestMapping(path = "/insertDrvFlow/DaysAgo")
    public void insertDrvFlowDaysAgo(@RequestParam("days") int days) {
        everyDayScheduleService.getDrvFlowDayData(days);
    }

    /**
     * 导入全量的驾驶证业务流水索引
     */
    @RequestMapping(path = "/insertDrvFlow/All")
    public void insertDrvFlowIndexAll() {
        everyDayScheduleService.getDrvFlowIndexAll();
    }

    @RequestMapping(path = "/insertVehIndex/Extra")
    public void insertVehIndexExtra() {
        vehicleIndexService.listVehIndexExtra();
    }


}
